"use strict";
/**
 * @warning internal.ts is some apis used internally only in fib-typify, we expose them
 * for `bin/*.js`, never dependes it in you App.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.watch = void 0;
const fs = require("fs");
const path = require("path");
const host_1 = require("./ts/host");
const { fixNonAbsolutePath, mkdirp } = require('../core/_utils');
const { formatAndPrintDiagnostic } = require('../core/ts-apis/diagnostic');
function isWatchValid() {
    return typeof fs.watchFile === 'function';
}
function watch(rootFileNames, options, workDir = process.cwd()) {
    if (!isWatchValid()) {
        throw new Error(`unsupported watch in this fibjs's version! fibjs >= 0.31.0 required!`);
    }
    const { services, files } = (0, host_1.createLanguageServiceFromFiles)({
        rootFileNames,
        options
    });
    // Now let's watch the files
    rootFileNames.forEach(fileName => {
        // First time around, emit all files
        emitFile(fileName);
        // Add a watch on the file to handle next change
        fs.watchFile(fileName, { persistent: true, interval: 250 }, (curr, prev) => {
            // Check timestamp
            if (+curr.mtime <= +prev.mtime) {
                return;
            }
            // Update the version to signal a change in the file
            files[fileName].version++;
            // write the changes to disk
            emitFile(fileName);
        });
    });
    function emitFile(fileName) {
        let output = services.getEmitOutput(fileName);
        if (!output.emitSkipped) {
            // console.log(`Emitting ${fileName}`);
        }
        else {
            // console.log(`Emitting ${fileName} failed`);
            logErrors(fileName);
        }
        output.outputFiles.forEach(o => {
            const fullp = fixNonAbsolutePath(o.name, workDir);
            mkdirp(path.dirname(fullp));
            fs.writeTextFile(fullp, o.text);
        });
    }
    function logErrors(fileName) {
        const allDiagnostics = services
            .getCompilerOptionsDiagnostics()
            .concat(services.getSyntacticDiagnostics(fileName))
            .concat(services.getSemanticDiagnostics(fileName));
        allDiagnostics.forEach(diagnostic => {
            formatAndPrintDiagnostic(diagnostic);
        });
    }
}
exports.watch = watch;
