"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLanguageServiceFromFiles = void 0;
const ts = require("typescript");
const fs = require("fs");
function createLanguageServiceFromFiles({ rootFileNames, options }) {
    const files = {};
    // initialize the list of files
    rootFileNames.forEach(fileName => {
        files[fileName] = { version: 0 };
    });
    const servicesHost = {
        getScriptFileNames: () => rootFileNames,
        getScriptVersion: fileName => files[fileName] && files[fileName].version.toString(),
        getScriptSnapshot: fileName => {
            if (!fs.exists(fileName)) {
                return undefined;
            }
            return ts.ScriptSnapshot.fromString(fs.readFile(fileName).toString());
        },
        getCurrentDirectory: () => process.cwd(),
        getCompilationSettings: () => options,
        getDefaultLibFileName: options => ts.getDefaultLibFilePath(options),
        fileExists: ts.sys.fileExists,
        readFile: ts.sys.readFile,
        readDirectory: ts.sys.readDirectory,
        directoryExists: ts.sys.directoryExists,
        getDirectories: ts.sys.getDirectories,
        // writeFile: ts.sys.writeFile
    };
    const services = ts.createLanguageService(servicesHost);
    return {
        services,
        files
    };
}
exports.createLanguageServiceFromFiles = createLanguageServiceFromFiles;
